% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Ph.D. Course "Intelligence for Embedded Systems"
% Prof. Manuel Roveri
% Politecnico di Milano, Italy
% Jan, 2015
% 
% ----- "From Metrology to digital data"
% -> Example 2_B) Precision 
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
clc;

% Consider the additive signal plus noise model x = x0 + eta
% where x0 is the ideal, noise free unknown measurement value 
% and eta is an i.i.d. zero mean random variable

% Parameters of the Gaussian Distribution
x0 = 0.5;                % Ideal, noise free unknown value
std_noise = 0.5;          % Standar deviation of the noise

lambda = 2; % Parameter of the exponential distribution

n = 10000;              % Number of acquired samples
confidence = '0.99';    % Confidence value for the normal distribution {'0.95','0.99'} 

% Generated samples with Gaussian noise
X = x0 + std_noise * randn(n,1);

% Sample mean to estimate the expected value
fprintf('\nCASE 1: GAUSSIAN DISTRIBUTION\n');

Mean  = mean(X);
fprintf('Sample mean = %.2f\n',Mean);

Standard_deviation = std(X);
fprintf('Sample Standard Deviation = %.2f\n',Standard_deviation);

switch (confidence)
    case '0.95'
        CI = [x0-2*std_noise,x0+2*std_noise];
        confidence_tcheb = 0.750;
    case '0.99'
        CI = [x0-3*std_noise,x0+3*std_noise];
        confidence_tcheb = 0.889;
    otherwise
        disp('');
end

counter = 0;
for nexp = 1: n
    X = x0 + std_noise * randn(1); 
    if (X > CI (1) && X < CI(2))
        counter = counter + 1;
    end
end

fprintf('\nConfidence Interval = [%.2f,%.2f]\n',CI(1),CI(2));
fprintf('Expected confidence Gaussian = %s\n', confidence);
fprintf('Expected confidence Tcheb = %.3f\n', confidence_tcheb);
fprintf('Ratio of examples within CI %.3f\n', counter/n);

fprintf('\nCASE 2: FOR EXPONENTIAL DISTRIBUTION\n');

pd = makedist('Exponential');
pd.mu = 1/lambda;


x0=1/lambda;
std_noise = sqrt(1 / lambda^2);

switch (confidence)
    case '0.95'
        CI = [x0-2*std_noise,x0+2*std_noise];
        confidence_tcheb = 0.750;
    case '0.99'
        CI = [x0-3*std_noise,x0+3*std_noise]; 
        
    otherwise
        disp('');
end

fprintf('Confidence Interval = [%.2f,%.2f]\n',CI(1),CI(2));
fprintf('Expected confidence Gaussian = %s\n', confidence);
fprintf('Expected confidence Tcheb = %.3f\n', confidence_tcheb);

counter = 0;
for nexp = 1: n
    X = random(pd,1,1); % To keep std equals to 1
    
    if (X > CI (1) && X < CI(2))
        counter = counter + 1;
    end
    X_vector(nexp) = X;
end

fprintf('Ratio of examples within CI %.3f\n', counter/n);
figure (1),hist(X_vector,100);
title('Empirical exponential distribution');
