% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Ph.D. Course "Intelligence for Embedded Systems"
% Prof. Manuel Roveri
% Politecnico di Milano, Italy
% Jan, 2015
% 
% ----- "From Metrology to digital data"
% -> Example 2_C) Deterministic vs Stochastic representation of Data 
%
% Modify the number of bits and/or the std of the noise
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
clc

% Consider the additive signal plus noise model x = x0 + eta
% where x0 is the deal, noise free unknown measurement value 
% and eta is an i.i.d. zero mean random variable

a = 0;                % Minimum value for the signal
b = 10;                 % Maximum value for the signal
n = 100;              % Number of acquired samples
nbit = 5;               % Number of bits
ncodeword = 2^nbit;     % Number of available codewords
lambda = 3;             % Inducing the confidence value (precision)
sigma_noise = 0.1;      % Std of the noise


points = linspace(a,b,ncodeword);    % Interval points
binary_codeword = zeros(2^nbit,nbit);
for i = 1 : 2^nbit
    binary_codeword(i,1:nbit) =  fliplr(de2bi(i-1,nbit));
end

X = a + (b-a) * rand(n,1);           % Generated points (uniform distribution)

% %%%% NOISE FREE SCENARIO %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp ('Noise Free Scenario');

% Coding of the acquisition
binary_X = zeros(n,nbit);
for i = 1 : n
    
    abs_diff = abs(X(i) - points);
    curr_index = find(abs_diff==min(abs_diff));
    quant_X (i) = points(curr_index);
    error (i) = X(i)-quant_X (i);
    binary_X (i,:) =  binary_codeword(curr_index,:);
    
end

% Show true value, quantized value and the corresponding error
figure (1),
plot(X,'*')
hold on
plot(quant_X,'rd')
xlabel ('Sample');
ylabel ('Value');
legend('True value','Quantized value');
title('Noise Free Scenario');
for i = 1 : n
    line([i,i],[X(i), quant_X(i)],'LineStyle',':');
end

% Estimated mean and std error
estimated_mean_error = mean(error)
estimated_std_error = std(error)

expected_mean_error = 0
expected_std_error = sqrt((points(2)-points(1))^2/12)

% %%%% NOISE AFFECTED SCENARIO %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp ('Noise Affected Scenario');

% Let us consider a fixed point x0
x0 = points (2);

noise = randn(n,1) * sigma_noise;   % Gaussian noise is considered
noisy_X0 = x0 + noise;

independent_points = (b - a) / ( 2 * lambda * sigma_noise) + 1
ncodeword

number_of_significant_bit = ceil(log2(independent_points))
